@@ -31,12 +31,15 @@
 #include <linux/mii.h>
 #include <linux/version.h>
 #include <linux/rtnetlink.h>
+#include <fcntl.h>
 
 #include "nm-utils.h"
 #include "nm-setting-wired.h"
 
 #include "nm-core-utils.h"
 
+extern char *if_indextoname (unsigned int __ifindex, char *__ifname);
+
 /******************************************************************
  * ethtool
  ******************************************************************/
@@ -60,7 +63,7 @@ ethtool_get (const char *name, gpointer edata)
 	nm_utils_ifname_cpy (ifr.ifr_name, name);
 	ifr.ifr_data = edata;
 
-	fd = socket (PF_INET, SOCK_DGRAM, 0);
+	fd = socket (PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (fd < 0) {
 		nm_log_err (LOGD_PLATFORM, "ethtool: Could not open socket.");
 		return FALSE;
@@ -144,8 +147,7 @@ nmp_utils_ethtool_get_permanent_address (const char *ifname,
 		struct ethtool_perm_addr e;
 		guint8 _extra_data[NM_UTILS_HWADDR_LEN_MAX + 1];
 	} edata;
-	static const guint8 zeros[NM_UTILS_HWADDR_LEN_MAX] = { 0 };
-	static guint8 ones[NM_UTILS_HWADDR_LEN_MAX] = { 0 };
+	guint i;
 
 	if (!ifname)
 		return FALSE;
@@ -157,18 +159,23 @@ nmp_utils_ethtool_get_permanent_address (const char *ifname,
 	if (!ethtool_get (ifname, &edata.e))
 		return FALSE;
 
-	g_assert (edata.e.size <= NM_UTILS_HWADDR_LEN_MAX);
-
-	/* Some drivers might return a permanent address of all zeros.
-	 * Reject that (rh#1264024) */
-	if (memcmp (edata.e.data, zeros, edata.e.size) == 0)
+	if (edata.e.size > NM_UTILS_HWADDR_LEN_MAX)
+		return FALSE;
+	if (edata.e.size < 1)
 		return FALSE;
 
-	/* Some drivers return a permanent address of all ones. Reject that too */
-	if (G_UNLIKELY (ones[0] != 0xFF))
-		memset (ones, 0xFF, sizeof (ones));
-	if (memcmp (edata.e.data, ones, edata.e.size) == 0)
+	if (NM_IN_SET (edata.e.data[0], 0, 0xFF)) {
+		/* Some drivers might return a permanent address of all zeros.
+		 * Reject that (rh#1264024)
+		 *
+		 * Some drivers return a permanent address of all ones. Reject that too */
+		for (i = 1; i < edata.e.size; i++) {
+			if (edata.e.data[0] != edata.e.data[i])
+				goto not_all_0or1;
+		}
 		return FALSE;
+	}
+not_all_0or1:
 
 	memcpy (buf, edata.e.data, edata.e.size);
 	*length = edata.e.size;
@@ -338,7 +345,7 @@ nmp_utils_mii_supports_carrier_detect (const char *ifname)
 	if (!nmp_utils_device_exists (ifname))
 		return FALSE;
 
-	fd = socket (PF_INET, SOCK_DGRAM, 0);
+	fd = socket (PF_INET, SOCK_DGRAM | SOCK_CLOEXEC, 0);
 	if (fd < 0) {
 		nm_log_err (LOGD_PLATFORM, "mii: couldn't open control socket (%s)", ifname);
 		return FALSE;
@@ -433,14 +440,35 @@ nmp_utils_device_exists (const char *name)
 	return g_file_test (sysdir, G_FILE_TEST_EXISTS);
 }
 
-guint
-nmp_utils_ip_config_source_to_rtprot (NMIPConfigSource source)
+NMIPConfigSource
+nmp_utils_ip_config_source_from_rtprot (guint8 rtprot)
 {
-	switch (source) {
-	case NM_IP_CONFIG_SOURCE_UNKNOWN:
+	return ((int) rtprot) + 1;
+}
+
+NMIPConfigSource
+nmp_utils_ip_config_source_round_trip_rtprot (NMIPConfigSource source)
+{
+	/* when adding a route to kernel for a give @source, the resulting route
+	 * will be put into the cache with a source of NM_IP_CONFIG_SOURCE_RTPROT_*.
+	 * This function returns that. */
+	return nmp_utils_ip_config_source_from_rtprot (nmp_utils_ip_config_source_coerce_to_rtprot (source));
+}
+
+guint8
+nmp_utils_ip_config_source_coerce_to_rtprot (NMIPConfigSource source)
+{
+	/* when adding a route to kernel, we coerce the @source field
+	 * to rtm_protocol. This is not lossless as we map different
+	 * source values to the same RTPROT uint8 value. */
+	if (source <= NM_IP_CONFIG_SOURCE_UNKNOWN)
 		return RTPROT_UNSPEC;
+
+	if (source <= _NM_IP_CONFIG_SOURCE_RTPROT_LAST)
+		return source - 1;
+
+	switch (source) {
 	case NM_IP_CONFIG_SOURCE_KERNEL:
-	case NM_IP_CONFIG_SOURCE_RTPROT_KERNEL:
 		return RTPROT_KERNEL;
 	case NM_IP_CONFIG_SOURCE_DHCP:
 		return RTPROT_DHCP;
@@ -453,18 +481,32 @@ nmp_utils_ip_config_source_to_rtprot (NMIPConfigSource source)
 }
 
 NMIPConfigSource
-nmp_utils_ip_config_source_from_rtprot (guint rtprot)
+nmp_utils_ip_config_source_coerce_from_rtprot (NMIPConfigSource source)
 {
-	switch (rtprot) {
-	case RTPROT_UNSPEC:
+	/* When we receive a route from kernel and put it into the platform cache,
+	 * we preserve the protocol field by converting it to a NMIPConfigSource
+	 * via nmp_utils_ip_config_source_from_rtprot().
+	 *
+	 * However, that is not the inverse of nmp_utils_ip_config_source_coerce_to_rtprot().
+	 * Instead, to go back to the original value, you need another step:
+	 *   nmp_utils_ip_config_source_coerce_from_rtprot (nmp_utils_ip_config_source_from_rtprot (rtprot)).
+	 *
+	 * This might partly restore the original source value, but of course that
+	 * is not really possible because nmp_utils_ip_config_source_coerce_to_rtprot()
+	 * is not injective.
+	 * */
+	switch (source) {
+	case NM_IP_CONFIG_SOURCE_RTPROT_UNSPEC:
 		return NM_IP_CONFIG_SOURCE_UNKNOWN;
-	case RTPROT_KERNEL:
-		return NM_IP_CONFIG_SOURCE_RTPROT_KERNEL;
-	case RTPROT_REDIRECT:
+
+	case NM_IP_CONFIG_SOURCE_RTPROT_KERNEL:
+	case NM_IP_CONFIG_SOURCE_RTPROT_REDIRECT:
 		return NM_IP_CONFIG_SOURCE_KERNEL;
-	case RTPROT_RA:
+
+	case NM_IP_CONFIG_SOURCE_RTPROT_RA:
 		return NM_IP_CONFIG_SOURCE_RDISC;
-	case RTPROT_DHCP:
+
+	case NM_IP_CONFIG_SOURCE_RTPROT_DHCP:
 		return NM_IP_CONFIG_SOURCE_DHCP;
 
 	default:
@@ -472,3 +514,130 @@ nmp_utils_ip_config_source_from_rtprot (guint rtprot)
 	}
 }
 
+const char *
+nmp_utils_ip_config_source_to_string (NMIPConfigSource source, char *buf, gsize len)
+{
+	const char *s = NULL;
+	nm_utils_to_string_buffer_init (&buf, &len); \
+
+	if (!len)
+		return buf;
+
+	switch (source) {
+	case NM_IP_CONFIG_SOURCE_UNKNOWN:         s = "unknown"; break;
+
+	case NM_IP_CONFIG_SOURCE_RTPROT_UNSPEC:   s = "rt-unspec"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_REDIRECT: s = "rt-redirect"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_KERNEL:   s = "rt-kernel"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_BOOT:     s = "rt-boot"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_STATIC:   s = "rt-static"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_DHCP:     s = "rt-dhcp"; break;
+	case NM_IP_CONFIG_SOURCE_RTPROT_RA:       s = "rt-ra"; break;
+
+	case NM_IP_CONFIG_SOURCE_KERNEL:          s = "kernel"; break;
+	case NM_IP_CONFIG_SOURCE_SHARED:          s = "shared"; break;
+	case NM_IP_CONFIG_SOURCE_IP4LL:           s = "ipv4ll"; break;
+	case NM_IP_CONFIG_SOURCE_PPP:             s = "ppp"; break;
+	case NM_IP_CONFIG_SOURCE_WWAN:            s = "wwan"; break;
+	case NM_IP_CONFIG_SOURCE_VPN:             s = "vpn"; break;
+	case NM_IP_CONFIG_SOURCE_DHCP:            s = "dhcp"; break;
+	case NM_IP_CONFIG_SOURCE_RDISC:           s = "rdisc"; break;
+	case NM_IP_CONFIG_SOURCE_USER:            s = "user"; break;
+	default:
+		break;
+	}
+
+	if (source >= 1 && source <= 0x100) {
+		if (s)
+			g_snprintf (buf, len, "%s", s);
+		else
+			g_snprintf (buf, len, "rt-%d", ((int) source) - 1);
+	} else {
+		if (s)
+			g_strlcpy (buf, s, len);
+		else
+			g_snprintf (buf, len, "(%d)", source);
+	}
+	return buf;
+}
+
+/**
+ * nmp_utils_sysctl_open_netdir:
+ * @ifindex: the ifindex for which to open "/sys/class/net/%s"
+ * @ifname_guess: (allow-none): optional argument, if present used as initial
+ *   guess as the current name for @ifindex. If guessed right,
+ *   it saves an addtional if_indextoname() call.
+ * @out_ifname: (allow-none): if present, must be at least IFNAMSIZ
+ *   characters. On success, this will contain the actual ifname
+ *   found while opening the directory.
+ *
+ * Returns: a negative value on failure, on success returns the open fd
+ *   to the "/sys/class/net/%s" directory for @ifindex.
+ */
+int
+nmp_utils_sysctl_open_netdir (int ifindex,
+                              const char *ifname_guess,
+                              char *out_ifname)
+{
+	#define SYS_CLASS_NET "/sys/class/net/"
+	const char *ifname = ifname_guess;
+	char ifname_buf_last_try[IFNAMSIZ];
+	char ifname_buf[IFNAMSIZ];
+	guint try_count = 0;
+	char sysdir[NM_STRLEN (SYS_CLASS_NET) + IFNAMSIZ] = SYS_CLASS_NET;
+	char fd_buf[256];
+	ssize_t nn;
+
+	g_return_val_if_fail (ifindex >= 0, -1);
+
+	ifname_buf_last_try[0] = '\0';
+
+	for (try_count = 0; try_count < 10; try_count++, ifname = NULL) {
+		nm_auto_close int fd_dir = -1;
+		nm_auto_close int fd_ifindex = -1;
+		int fd;
+
+		if (!ifname) {
+			ifname = if_indextoname (ifindex, ifname_buf);
+			if (!ifname)
+				return -1;
+		}
+
+		nm_assert (nm_utils_iface_valid_name (ifname));
+
+		if (g_strlcpy (&sysdir[NM_STRLEN (SYS_CLASS_NET)], ifname, IFNAMSIZ) >= IFNAMSIZ)
+			g_return_val_if_reached (-1);
+
+		/* we only retry, if the name changed since previous attempt.
+		 * Hence, it is extremely unlikely that this loop runes until the
+		 * end of the @try_count. */
+		if (nm_streq (ifname, ifname_buf_last_try))
+			return -1;
+		strcpy (ifname_buf_last_try, ifname);
+
+		fd_dir = open (sysdir, O_DIRECTORY | O_CLOEXEC);
+		if (fd_dir < 0)
+			continue;
+
+		fd_ifindex = openat (fd_dir, "ifindex", O_CLOEXEC);
+		if (fd_ifindex < 0)
+			continue;
+
+		nn = nm_utils_fd_read_loop (fd_ifindex, fd_buf, sizeof (fd_buf) - 2, FALSE);
+		if (nn <= 0)
+			continue;
+		fd_buf[nn] = '\0';
+
+		if (ifindex != _nm_utils_ascii_str_to_int64 (fd_buf, 10, 1, G_MAXINT, -1))
+			continue;
+
+		if (out_ifname)
+			strcpy (out_ifname, ifname);
+
+		fd = fd_dir;
+		fd_dir = -1;
+		return fd;
+	}
+
+	return -1;
+}
